import sys
import threading

from tests.webserver import app

import logging

log = logging.getLogger('werkzeug')
log.disabled = True

def flaskserver():
    app.run(host='127.0.0.1', port=5000, debug=False)


th = threading.Thread(target=flaskserver, args=(), daemon=True)
th.start()

sys.path.append("../pyourls3")



import requests
import urllib3

import pyourls3
from pyourls3 import exceptions
import json


class Yourls:
    """
    Base class for Pyourls3

    :param addr: required, string. The base address that the YOURLS installation resides at.
    :param user: string. Username if not using key authorisation
    :param passwd: required if using user, string. Password for username/password auth.
    :param key: string. Can only be used if user and password are not specified.
    """

    def __init__(self, addr, user=None, passwd=None, key=None):

        if not addr:
            raise exceptions.Pyourls3ParamError("API URL")

        scheme, _, _, _, _, _, _ = urllib3.util.parse_url(addr)  # checking for a protocol in the URL

        if not scheme:
            raise exceptions.Pyourls3ParamError("addr")

        self.is_using_signature_auth = False

        if user is None or passwd is None:
            if key is None:
                raise exceptions.Pyourls3ParamError("username and password or signature")
            else:
                if type(key) is not str:
                    raise exceptions.Pyourls3ParamError("key")
                self.is_using_signature_auth = True
                self.global_args = {"signature": key}
        else:
            if type(user) is not str:
                raise exceptions.Pyourls3ParamError("user")
            elif type(passwd) is not str:
                raise exceptions.Pyourls3ParamError("passwd")
            self.global_args = {"username": user, "password": passwd}

        self.global_args["format"] = "json"

        if addr[-1] != "/":  # I like trailing slashes
            addr += "/"

        self.api_endpoint = addr + "yourls-api.php"

        self.session = requests.session()
        self.session.headers.update = {"user-agent": "pyourls3/{}".format(pyourls3.version)}

        if self.session.post(self.api_endpoint, data={**self.global_args}).status_code == 403:
            raise exceptions.Pyourls3APIError("forbidden", 403)

    def shorten(self, url, keyword=None, title=None):
        """
        Sends an API request to shorten a specified URL.

        :param url: required, string. URL to be shortened.
        :param keyword: string. Custom alias for the URL
        :param title: string. Custom title for  the URL.
        :return: dictionary. Full JSON response from the API, parsed into a dict

        :raises: pyourls3.exceptions.Pyourls3ParamError, pyourls3.exceptions.Pyourls3HTTPError,
          pyourls3.exceptions.Pyourls3APIError, pyourls3.exceptions.Pyourls3URLAlreadyExistsError
        """

        if not url:
            raise exceptions.Pyourls3ParamError("url")

        specific_args = {"action": "shorturl", "url": url}

        if keyword is not None:
            specific_args["keyword"] = keyword

        if title is not None:
            specific_args["title"] = title

        r = requests.post(self.api_endpoint, data={**self.global_args, **specific_args})
        try:
            j = r.json()
        except json.decoder.JSONDecodeError:
            raise exceptions.Pyourls3HTTPError(r.status_code, self.api_endpoint)

        if not j["status"] == "success":
            if j["code"] == "error:url":
                raise exceptions.Pyourls3URLAlreadyExistsError(url)
            else:
                raise exceptions.Pyourls3APIError(j["message"], j["code"])

        return j

    def expand(self, url):
        """
        Expands a specified URL or alias into it's full form.

        :param url: required, string. URL or alias for shortened link.
        :return: string. Expanded URL.
        """

        if not url:
            raise exceptions.Pyourls3ParamError("url")

        specific_args = {"action": "expand", "shorturl": url}

        r = requests.post(self.api_endpoint, data={**self.global_args, **specific_args})
        try:
            j = r.json()
        except json.decoder.JSONDecodeError:
            raise exceptions.Pyourls3HTTPError(r.status_code, self.api_endpoint)

        if not j["message"] == "success":
            raise exceptions.Pyourls3APIError(j["message"].split(": ")[1], j["code"])  # message returns "error: blah"

        return j["longurl"]

    def stats(self):
        """
        Returns the overall installation stats.

        :return: string. Partial JSON response returned by the API.

        :raises: pyourls3.exceptions.Pyourls3HTTPError
        """

        specific_args = {"action": "stats"}

        r = requests.post(self.api_endpoint, data={**self.global_args, **specific_args})
        try:
            j = r.json()
        except json.decoder.JSONDecodeError:
            raise exceptions.Pyourls3HTTPError(r.status_code, self.api_endpoint)

        return j["stats"]

    def url_stats(self, url):
        """
        Detailed stats about a specifc URL or alias.

        :param url: required, string. URL or alias of target redirect.
        :return: dict. Partial JSON response, parsed into a dict.

        :raises: pyourls3.exceptions.Pyourls3HTTPError, pyourls3.exceptions.Pyourls3APIError
        """

        if not url:
            raise exceptions.Pyourls3ParamError("url")

        specific_args = {"action": "url-stats", "shorturl": url}

        r = requests.post(self.api_endpoint, data={**self.global_args, **specific_args})
        try:
            j = r.json()
        except json.decoder.JSONDecodeError:
            raise exceptions.Pyourls3HTTPError(r.status_code, self.api_endpoint)

        if not j["message"] == "success":
            raise exceptions.Pyourls3APIError(j["message"].split(": ")[1], j["code"])  # message returns "error: blah"

        return j["link"]


from pyourls3 import client, exceptions
import pytest
import tests.sharevar


# initialisation tests
def test_i_url_exists():
    with pytest.raises(exceptions.Pyourls3ParamError):
        client.Yourls("")


def test_i_url_schema():
    with pytest.raises(exceptions.Pyourls3ParamError):
        client.Yourls("example.com")


def test_i_url_formatting():
    url = "https://example.com"
    c = client.Yourls(url, key="12345")
    assert c.api_endpoint != url + "yourls-api.php"
    assert c.api_endpoint == url + "/yourls-api.php"


def test_i_credentials_present():
    with pytest.raises(exceptions.Pyourls3ParamError):
        client.Yourls("http://example.com")


def test_i_credential_format():
    with pytest.raises(exceptions.Pyourls3ParamError):
        client.Yourls("http://example.com", key=123456789)
    with pytest.raises(exceptions.Pyourls3ParamError):
        client.Yourls("http://example.com", user="hello", passwd=12345)
    with pytest.raises(exceptions.Pyourls3ParamError):
        client.Yourls("http://example.com", user=12345)


def test_i_user_passwd():
    client.Yourls("http://example.com", user="username", passwd="password")


def test_i_api_key():
    client.Yourls("http://example.com", key="123456789")


def test_i_user_agent():
    inst = client.Yourls("http://example.com", key="1234")
    assert "user-agent" in inst.session.headers.update


def test_i_creds_correct():
    tests.sharevar.modifier = "badauth"
    with pytest.raises(exceptions.Pyourls3APIError):
        client.Yourls(tests.sharevar.standard_url, key="12345")
    tests.sharevar.modifier = None


# This file exists only to transfer variables between the Flask server running in it.
# Say what you want, this is useful and I am going to continue to use it. Shaddup.

standard_url = "http://127.0.0.1:5000/"


last_request = None
modifier = None



from pyourls3 import client, exceptions
import pytest
import tests
import tests.sharevar

# tests for the url specific stats function
@pytest.fixture(scope="module")
def instance():
    c = client.Yourls(tests.sharevar.standard_url, key="12345")
    yield c
    tests.sharevar.modifier = None


def test_e_url_exists(instance):
    with pytest.raises(exceptions.Pyourls3ParamError):
        instance.url_stats("")


def test_e_request_params_correct(instance):
    u = "https://www.google.com"
    instance.url_stats(u)
    assert "shorturl" in tests.sharevar.last_request
    assert tests.sharevar.last_request["shorturl"] == u


def test_e_garbled_json(instance):
    tests.sharevar.modifier = "garbledjson"
    with pytest.raises(exceptions.Pyourls3HTTPError):
        instance.url_stats("https://www.google.com")


def test_e_generic_error(instance):
    tests.sharevar.modifier = "error"
    with pytest.raises(exceptions.Pyourls3APIError):
        instance.url_stats("https://www.google.com")



from pyourls3 import client, exceptions
import pytest
import tests
import tests.sharevar

# tests for  the shorten function, utilises a Flask WSGI server. Data is passed from that server through sharevar.py
# it is crucial that if the modifier is used, it is RESET TO NONE when the test is complete. This can be done by using
# the fixture "instance"


@pytest.fixture(scope="module")
def instance():
    c = client.Yourls(tests.sharevar.standard_url, key="12345")
    yield c
    tests.sharevar.modifier = None


def test_s_url_exists(instance):
    with pytest.raises(exceptions.Pyourls3ParamError):
        instance.shorten("")


def test_s_keyword(instance):
    kwd = "keyword goes here"
    instance.shorten("https://www.google.com", keyword=kwd)
    assert "keyword" in tests.sharevar.last_request
    assert tests.sharevar.last_request["keyword"] == kwd


def test_s_title(instance):
    title = "this should be a title"
    instance.shorten("https://www.google.com", title=title)
    assert "title" in tests.sharevar.last_request
    assert tests.sharevar.last_request["title"] == title


def test_s_garbled_json(instance):
    tests.sharevar.modifier = "garbledjson"
    with pytest.raises(exceptions.Pyourls3HTTPError):
        instance.shorten("https://www.google.com")


def test_s_url_error(instance):
    tests.sharevar.modifier = "urlerror"
    with pytest.raises(exceptions.Pyourls3URLAlreadyExistsError):
        instance.shorten("https://www.google.com")


def test_s_other_error(instance):
    tests.sharevar.modifier = "othererror"
    with pytest.raises(exceptions.Pyourls3APIError):
        instance.shorten("https://www.google.com")



from pyourls3 import client, exceptions
import pytest
import tests
import tests.sharevar


# tests for the stats function
@pytest.fixture(scope="module")
def instance():
    c = client.Yourls(tests.sharevar.standard_url, key="12345")
    yield c
    tests.sharevar.modifier = None


def test_s_garbled_json(instance):
    tests.sharevar.modifier = "garbledjson"
    with pytest.raises(exceptions.Pyourls3HTTPError):
        instance.stats()



import flask
import tests.sharevar

app = flask.Flask(__name__)


@app.route("/yourls-api.php", methods=["POST"])
def proc_cron():

    if "action" not in flask.request.form:
        if tests.sharevar.modifier == "badauth":
            response = flask.Response('{"message": "Invalid username or password","errorCode": 403,"callback": ""}',
                                      status=403)
        else:
            response = flask.Response('{"errorCode": 400,"message": "Unknown or missing "action" parameter"}',
                                      status=400)

        response.headers["content-type"] = "application/json"
        return response
    else:
        option = flask.request.form["action"]

    tests.sharevar.last_request = flask.request.form

    if option == "shorturl":
        if tests.sharevar.modifier == "garbledjson":
            response = flask.Response("this is not valid json")

        elif tests.sharevar.modifier == "urlerror":
            response = flask.Response('{"status": "error", "code": "error:url", "message": "message goes here"}')

        elif tests.sharevar.modifier == "othererror":
            response = flask.Response('{"status": "error", "code": "something", "message": "message goes here"}')

        else:
            response = flask.Response('{"url": {"keyword": "ozh","url": "http://ozh.org","title": "Ozh RICHARD \u00ab o'
                                      'zh.org","date": "2014-10-24 16:01:39","ip": "127.0.0.1"},"status": "success","me'
                                      'ssage": "http://ozh.org added to database","title": "Ozh RICHARD \u00ab ozh.org"'
                                      ',"shorturl": "http://sho.rt/1f","statusCode": 200}')  # this is just the example
            # response included with the YOURLS docs

        response.headers["content-type"] = "application/json"
        return response

    elif option == "expand":
        if tests.sharevar.modifier == "garbledjson":
            response = flask.Response("this is not valid json")

        elif tests.sharevar.modifier == "error":
            response = flask.Response('{"status": "error", "code": "something", "message": "error: message goes here"}')

        else:
            response = flask.Response('{"message": "success", "longurl": "https://www.example.com"}')

        response.headers["content-type"] = "application/json"
        return response

    elif option == "stats":
        if tests.sharevar.modifier == "garbledjson":
            response = flask.Response("this is not valid json")

        else:
            response = flask.Response('{"stats": {"total_links": "3","total_clicks": "2"},"statusCode": 200,"message":'
                                      ' "success"}')

        response.headers["content-type"] = "application/json"
        return response

    elif option == "url-stats":
        if tests.sharevar.modifier == "garbledjson":
            response = flask.Response("this is not valid json")

        elif tests.sharevar.modifier == "error":
            response = flask.Response('{"status": "error", "code": "something", "message": "error: message goes here"}')

        else:
            response = flask.Response('{"message": "success", "link": {"shorturl": "durdehurrhurr","url'
                                      '": "hurdedurrdurr","title": "title goes here","timestamp": "2019'
                                      '-09-19 19:10:42","ip": "xxx.xxx.xxx.xxx","clicks": "2"}}')

        response.headers["content-type"] = "application/json"
        return response

if __name__ == "__main__":
    app.run()



class Pyourls3Error(Exception):
    """Base exception for Pyourls3.

    :param msg: string, full exception text.
    """

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg


class Pyourls3ParamError(Pyourls3Error):
    """Raised when a function parameter is incorrect

    :param param: string, parameter name that caused the issue
    """

    def __init__(self, param):
        self.param = param

    def __str__(self):
        return "Parameter '{}' is either missing and required, or is malformed.".format(self.param)


class Pyourls3HTTPError(Pyourls3Error):
    """Raised in the event that a HTTP error is caused

    :param status_code: string, HTTP status code
    :param url: string, URL that generated the HTTP error
    """

    def __init__(self, status_code, url):
        self.status_code = status_code
        self.url = url

    def __str__(self):
        return "A HTTP error was encountered during a request to {} - {}".format(self.url, self.status_code)


class Pyourls3APIError(Pyourls3Error):
    """Raised when the API raises an error and returns it through JSON

    :param message: string, message sent back by API
    :param code: string, status code sent back by the API
    """

    def __init__(self, message, code):
        self.message = message
        self.code = code

    def __str__(self):
        return "An error was returned from the server during a request - {} ({})".format(self.message, self.code)


class Pyourls3URLAlreadyExistsError(Pyourls3APIError):
    """Raised specifically when the API returns an error stating the URL already has a redirect created for it
    It's derived from an API error.

    :param url: URL that's already redirected to
    """

    def __init__(self, url):
        self.url = url

    def __str__(self):
        return "There is already a redirect to the specified URL ({}) in the database".format(self.url)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyourls3",
    version="1.0.1",
    author="Thomas Pain",
    author_email="pyourls3@tdpain.net",
    description="A Python 3 API wrapper for YOURLS",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://www.tdpain.net/projects/pyourls3",
    packages=["pyourls3"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
    install_requires=["requests",
                      "certifi>=2017.4.17",
                      "chardet<3.1.0,>=3.0.2",
                      "idna<2.9,>=2.5",
                      "urllib3!=1.25.0,!=1.25.1,<1.26,>=1.21.1"]
)